@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2019, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2025, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -31,17 +31,22 @@ module VNMMAD
             #   @param xpath_filer [String] to get the VM NICs
             #   @param deploy_id [String] refers to the VM in the hypervisor
             def initialize(vm_root, xpath_filter, deploy_id)
-                @vm_root      = vm_root
-                @deploy_id    = deploy_id
+                @vm_root   = vm_root
+                @deploy_id = deploy_id
 
-                @vm_info      = {}
+                @vm_info = {}
 
                 @deploy_id = nil if deploy_id == '-'
 
-                nics = VNMNetwork::Nics.new(hypervisor)
+                @nics       = VNMNetwork::Nics.new(hypervisor)
+                @nics_alias = VNMNetwork::Nics.new(hypervisor)
+
+                @pcis       = VNMNetwork::Nics.new(hypervisor)
+
+                return if xpath_filter.nil?
 
                 @vm_root.elements.each(xpath_filter) do |nic_element|
-                    nic = nics.new_nic
+                    nic = @nics.new_nic
 
                     nic_build_hash(nic_element, nic)
 
@@ -50,14 +55,30 @@ module VNMMAD
                         nic.get_tap(self)
                     end
 
-                    nics << nic
+                    @nics << nic
                 end
 
-                @nics = nics
+                @vm_root.elements.each('TEMPLATE/NIC_ALIAS') do |nic_element|
+                    nic = @nics_alias.new_nic
+
+                    nic_build_hash(nic_element, nic)
+
+                    @nics_alias << nic
+                end
+
+                pci_xpath_filter = xpath_filter.gsub(/\/NIC/,'/PCI')
+
+                @vm_root.elements.each(pci_xpath_filter) do |ne|
+                    nic = @pcis.new_nic
+
+                    nic_build_hash(ne, nic)
+
+                    @pcis << nic
+                end
             end
 
             # Iterator on each NIC of the VM
-            def each_nic(block)
+            def each_nic(&block)
                 return if @nics.nil?
 
                 @nics.each do |the_nic|
@@ -65,6 +86,48 @@ module VNMMAD
                 end
             end
 
+            # Iterator on each NIC_ALIAS of the VM
+            def each_nic_alias(&block)
+                return if @nics_alias.nil?
+
+                @nics_alias.each do |the_nic|
+                    block.call(the_nic)
+                end
+            end
+
+            # Iterator on each PCI of the VM
+            def each_pci(&block)
+                return if @pcis.nil?
+
+                @pcis.each do |the_nic|
+                    block.call(the_nic)
+                end
+            end
+
+            def each_nic_all(&block)
+                all_nics = @nics
+
+                if all_nics
+                    all_nics += @nics_alias
+                else
+                    all_nics = @nics_alias
+                end
+
+                return if all_nics.nil?
+
+                all_nics.each do |the_nic|
+                    block.call(the_nic)
+                end
+            end
+
+            def parent(nic_alias)
+                @nics.each do |the_nic|
+                    return the_nic if the_nic[:nic_id] == nic_alias[:parent_id]
+                end
+
+                nil
+            end
+
             # Access an XML Element of the VM
             #   @param element [String] element name
             #   @return [String] value of the element or nil if not found
@@ -93,6 +156,16 @@ module VNMMAD
                 "#{ds_location}/#{ds_id}/#{vm_id}"
             end
 
+            def changes
+                changes = {}
+
+                @vm_root.elements['TEMPLATE/VNET_UPDATE'].each do |elem|
+                    changes[elem.name.downcase.to_sym] = elem.text
+                end
+
+                changes
+            end
+
             private
 
             # Method to build the associated Hash from a NIC
