@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2019, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2025, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -14,8 +14,12 @@
 # limitations under the License.                                             #
 #--------------------------------------------------------------------------- #
 
-require 'shellwords'
+require 'json'
 require 'open3'
+require 'resolv'
+require 'shellwords'
+
+require_relative 'vf'
 
 ################################################################################
 # The VNMMAD module provides the basic abstraction to implement custom
@@ -23,6 +27,7 @@ require 'open3'
 #   - VNMNetwork with base classes and main functionality to manage Virtual Nets
 #   - SGIPTables a module with a SG implementation based in iptables/ipset
 ################################################################################
+# rubocop:disable Naming/VariableNumber
 module VNMMAD
 
     ############################################################################
@@ -33,12 +38,14 @@ module VNMMAD
 
         attr_reader :vm
 
+        include VirtualFunction
+
         # Creates new driver using:
         #   @param vm_tpl [String] XML String from oned
         #   @param xpath_filter [String] to get relevant NICs for the driver
         #   @param deploy_id [String]
         def initialize(vm_tpl, xpath_filter, deploy_id = nil)
-            @locking ||= false
+            @locking = false
 
             @vm = VNMNetwork::VM.new(REXML::Document.new(vm_tpl).root,
                                      xpath_filter, deploy_id)
@@ -56,32 +63,52 @@ module VNMMAD
         # Locking function to serialized driver operations if needed. Similar
         # to flock. File is created as /tmp/onevnm-<driver>-lock
         def lock
-            if @locking
-                driver_name = self.class.name.downcase
-                @locking_file = File.open("/tmp/onevnm-#{driver_name}-lock", 'w')
-                @locking_file.flock(File::LOCK_EX)
-            end
+            return unless @locking
+
+            driver_name = self.class.name.downcase
+            @locking_file = File.open("/tmp/onevnm-#{driver_name}-lock", 'w')
+            @locking_file.flock(File::LOCK_EX)
         end
 
         # Unlock driver execution mutex
         def unlock
-            if @locking
-                @locking_file.close
-            end
+            @locking_file.close if @locking
         end
 
         # Executes the given block on each NIC
         def process
-            blk = lambda do |nic|
-                add_nic_conf(nic)
-                add_bridge_conf(nic)
-                add_ovs_bridge_conf(nic)
-                add_ip_link_conf(nic)
+            @vm.each_nic do |nic|
+                nic_confs(nic)
+                yield(nic)
+            end
+        end
 
+        # Executes the given block on each NIC_ALIAS
+        def process_alias
+            @vm.each_nic_alias do |nic|
+                nic_confs(nic)
+                yield(nic)
+            end
+        end
+
+        # Executes the given block on each NIC
+        def process_all
+            @vm.each_nic do |nic|
+                nic_confs(nic)
                 yield(nic)
             end
 
-            @vm.each_nic(blk)
+            @vm.each_nic_alias do |nic|
+                nic_confs(nic)
+                yield(nic)
+            end
+        end
+
+        def nic_confs(nic)
+            add_nic_conf(nic)
+            add_bridge_conf(nic)
+            add_ovs_bridge_conf(nic)
+            add_ip_link_conf(nic)
         end
 
         # Parse network configuration and add it to the nic
@@ -155,11 +182,132 @@ module VNMMAD
             nic[conf_name] = nic_conf
         end
 
+        # Process nics and create bridges
+        def create_bridges
+            @bridges = list_bridges
+
+            process do |nic|
+                create_bridge(nic)
+            end
+        end
+
+        # Get hypervisor bridges
+        #   @return [Hash<String>] with the bridge names
+        def list_bridges
+            bridges = {}
+
+            ip_show_bridge =
+                `#{VNMNetwork::COMMANDS[:ip_unpriv]} link show type bridge`
+
+            ip_show_bridge.split("\n").each do |line|
+                next if line !~ /^[0-9]*:/
+
+                br_name = line.split(': ')[1]
+                bridges[br_name] = []
+
+                # rubocop:disable Layout/LineLength
+                ip_show_master =
+                    `#{VNMNetwork::COMMANDS[:ip_unpriv]} link show master #{br_name}`
+                # rubocop:enable Layout/LineLength
+
+                ip_show_master.split("\n").each do |l|
+                    next if l !~ /^[0-9]*:/
+
+                    bridges[br_name] << l.scan(/^\d+:\s([^:@]+)/)[0][0]
+                end
+            end
+
+            bridges
+        end
+
+        # Creates a bridge if it does not exists, and brings it up.
+        # This function IS FINAL, exits if action cannot be completed
+        #   @param nic [REXML::Element] @vm nic
+        def create_bridge(nic)
+            return if @bridges.key?(nic[:bridge])
+
+            LocalCommand.run_sh("#{command(:ip)} link add name " \
+                "'#{nic[:bridge]}' type bridge #{list_bridge_options(nic)}", :ok_rcs => 2)
+
+            @bridges[nic[:bridge]] = []
+
+            LocalCommand.run_sh("#{command(:ip)} " \
+                                    "link set '#{nic[:bridge]}' up")
+        end
+
+        # Deletes all vlan filters on bridge ports (VLAN range, 12bits 1-4094)
+        # VLAN 1 is preserved as it is configured by default.
+        def clean_vlan_filters(nic)
+            @bridges[nic[:bridge]].each do |dev|
+                LocalCommand.run_sh("#{command(:bridge)} vlan del dev #{dev}"\
+                            ' vid 2-4094', :ok_rcs => 2)
+            end
+        end
+
+        def set_vlan_filter(dev, pvid, vlans)
+            LocalCommand.run_sh("#{command(:bridge)} vlan add dev #{dev}"\
+                " vid #{pvid} pvid untagged", :ok_rcs => 2) if pvid
+
+            vlans.each do |vid|
+                LocalCommand.run_sh("#{command(:bridge)} vlan add dev #{dev}"\
+                    " vid #{vid}", :ok_rcs => 2)
+            end
+        end
+
+        # Reads config and return str with switches
+        #   @param nic [REXML::Element] @vm nic
+        #
+        # It reads both
+        #   * brctl options from :bridge_conf section
+        #   * ip-route2 bridge options from ip_bridge_conf section
+        #
+        # It tries to translate the options from brctl to ip-route2 format
+        # for the backward compatibility
+        def list_bridge_options(nic)
+            bridge_options = nic[:conf][:bridge_conf] || {}
+            ip_bridge_options = {}
+
+            # options transform table from brctl to ip-route2
+            brctl_to_ipbridge = {
+                :setageing     => :ageing_time,
+                :sethello      => :hello_time,
+                :setmaxage     => :max_age,
+                :stp           => :stp_state,
+                :setbridgeprio => :priority,
+                :setfd         => :forward_delay
+            }
+
+            # translate bridge_options to ip_bridge_options
+            bridge_options.each do |brctl_opt, brctl_val|
+                next unless brctl_to_ipbridge.include? brctl_opt
+
+                ip_bridge_options[brctl_to_ipbridge[brctl_opt]] = brctl_val
+            end
+
+            # merge, conf section `:ip_bridge_conf` has higher priority
+            ip_bridge_options.merge!(nic[:conf][:ip_bridge_conf]) \
+                if nic[:conf][:ip_bridge_conf]
+
+            bridge_options_str = ''
+            ip_bridge_options.each do |option, value|
+                case value
+                when true
+                    value = '1'
+                when false
+                    value = '0'
+                end
+
+                bridge_options_str << "#{option} #{value} "
+            end
+
+            bridge_options_str.strip
+        end
+
         # Returns a filter object based on the contents of the template
         #
         # @return SGDriver object
-        def self.filter_driver(vm_64, xpath_filter, deploy_id)
-            SGDriver.new(vm_64, xpath_filter, deploy_id)
+        def self.filter_driver(vm_64, xpath_filter, deploy_id, bridged = true)
+            SGDriver.new(vm_64, xpath_filter, deploy_id, bridged)
         end
 
         # Returns the associated command including sudo and other configuration
@@ -202,7 +350,7 @@ module VNMMAD
             return 0 if Dir["#{dir}/*"].empty?
 
             programs(dir).each do |file|
-                OpenNebula.log "Running #{file}"
+                OpenNebula::DriverLogger.log "Running #{file}"
 
                 cmd = "#{file} #{args.join(' ')}"
 
@@ -210,12 +358,22 @@ module VNMMAD
 
                 raise "Error running #{file}\n#{e}" unless s.exitstatus.zero?
 
-                OpenNebula.log o
+                OpenNebula::DriverLogger.log o
             end
 
             0
         end
 
+        def run_hooks_remote(args, stdin)
+            hostname = args[0]
+
+            cmd = "run-parts #{$PROGRAM_NAME}.d".gsub('/var/lib/one/remotes',
+                                                      '/var/tmp/one')
+            args.each {|arg| cmd << " --arg=\"#{arg}\"" }
+
+            SSHCommand.run(cmd, hostname, nil, stdin, 60)
+        end
+
         private
 
         # returns files sorted alphabetically
@@ -238,3 +396,4 @@ module VNMMAD
     end
 
 end
+# rubocop:enable Naming/VariableNumber
